// C i r c l e B o y . c p p 
// 		based fairly heavily on CircleToy.cpp (see comment below)
//			adapted by Yitzi Jacob (yitz@datalife.com)
//			circleBoy draws ellipsis that represent the contents of a file.
//			in it's current incarnation circleBoy queries all the files
//			on your boot volume larger than 7k and displays them in random
//			order .. (actually it's more like chaos.)
//
//		**note: it only displays the first 7k of the file currently
//				but that is enough to get the idea. 

// CircleToy.cpp
// A good tutorial application for the beginner.
// You may use this source code in any way you see fit.  It is in the
// public domain.
// Michael D. Crawford crawford@goingware.com
// GoingWare - Expert Software Development and Consulting
// http://www.goingware.com

#include <Application.h>
#include <Window.h>
#include <View.h>
#include <storage/VolumeRoster.h>
#include <storage/Volume.h>
#include <storage/Query.h>
#include <storage/Entry.h>
#include <storage/File.h>
#include <storage/Path.h>

typedef unsigned char* ybyte;

class CircleApp: public BApplication{
	public:
		CircleApp();
		virtual ~CircleApp();
};

class CircleWindow: public BWindow{
	public:
		CircleWindow();
		virtual ~CircleWindow();
		virtual bool QuitRequested();
};

class CircleView: public BView{
	public:
		unsigned char** vars;
		int m_vnum;
		BQuery que;
		char* filename;
		CircleView( BRect frame );
		virtual ~CircleView();		
		virtual void Draw( BRect updateRect );
		virtual void Pulse();
		void flagset();
};

int main( int argc, char **argv )
{
	BApplication *myApp = new CircleApp;

	myApp->Run();
	
	delete myApp;
	
	return 0;
} 

CircleApp::CircleApp()
	: BApplication( "application/x-circledraw" )
{
	BWindow *myWindow = new CircleWindow;
	myWindow->SetPulseRate(2000000); //2 seconds..
	myWindow->Show();
	
	return;
}

CircleApp::~CircleApp()
{
	return;
}

CircleWindow::CircleWindow()
	: BWindow( BRect( 100, 100, 300, 300 ),
										"CircleBoy",
										B_DOCUMENT_WINDOW,
										0 )
{
	BView *myView = new CircleView( Bounds() );
//	((CircleView*)myView)->flagset();//SetFlags(B_FULL_UPDATE_ON_RESIZE);

	AddChild( myView );
	
	return;
}

CircleWindow::~CircleWindow()
{
	return;
}

bool CircleWindow::QuitRequested()
{
	be_app->PostMessage( B_QUIT_REQUESTED );
	
	return true;
}

CircleView::CircleView( BRect frame )
	: BView( frame, "CircleView", B_FOLLOW_ALL_SIDES/*_RESIZABLE*/, B_WILL_DRAW )
{
	BVolumeRoster ros;
	BVolume vol;
	BEntry ent;
	BPath pat;
	
	filename = new char[128];
	ros.GetBootVolume(&vol);
	que.SetVolume(&vol);
	que.PushAttr("size");
	que.PushInt32(7000);
	que.PushOp(B_GT);
	if(que.Fetch()!=B_OK)
		exit(-1);
	BFile phil;
	unsigned char b[7000];
//	int actual;
	que.GetNextEntry(&ent);
	ent.GetPath(&pat);
	strcpy(filename, pat.Path());
	phil.SetTo(&ent/*"/boot/home/art.jpg"*/,B_READ_ONLY);
	phil.Read(&b,7000);

	//SetViewColor(B_TRANSPARENT_32_BIT);
	m_vnum = 7000/7;
	vars = new ybyte[m_vnum];

	for(int vnex = 0; vnex <m_vnum; vnex++)
	{
		vars[vnex] = new unsigned char[7];		

			for(int v = 0; v < 7; v++)
			{
				vars[vnex][v] = b[vnex+v];//char(rand());
			}
	}
	return;
}

CircleView::~CircleView()
{
	return;
}

void CircleView::Draw( BRect updateRect )
{

	flagset();
	Window()->Lock();
	SetOrigin(Bounds().right/3,Bounds().bottom/3);
for(int i=0;i<m_vnum;i++)
{
	BPoint p;
	p.y = vars[i][0] % int(Bounds().bottom);
	p.x = vars[i][1] % int(Bounds().right);
	
	//SetFlags(B_FRAME_EVENTS);//B_FULL_UPDATE_ON_RESIZE);
	
		SetHighColor(vars[i][2] % 256, vars[i][3] % 256, vars[i][4] % 256);
		FillEllipse( p, vars[i][5] % int(Bounds().right), vars[i][6] % int(Bounds().bottom) );		
}
	SetOrigin(10.0,10.0);
	DrawString(filename);

	Window()->Unlock();
	return;
}

void CircleView::flagset()
{
	SetFlags(B_FULL_UPDATE_ON_RESIZE | B_PULSE_NEEDED);
	return;
}

void CircleView::Pulse()
{
	BFile phil;
	BEntry ent;
	BPath pat;
	unsigned char b[7000];
//	int actual;
	if(que.GetNextEntry(&ent)==B_OK)
	{
		ent.GetPath(&pat);
		strcpy(filename, pat.Path());
		phil.SetTo(&ent/*"/boot/home/art.jpg"*/,B_READ_ONLY);
		phil.Read(&b,7000);

		for(int vnex = 0; vnex <m_vnum; vnex++)
		{
				for(int v = 0; v < 7; v++)
				{
					vars[vnex][v] = b[vnex+v];//char(rand());
				}
		}
		Invalidate();
	}
	return;	
}